﻿local Engine, Api, Locale, Settings = unpack(AshranBuddy)

-- Registration
local EventTimer = Engine:RegisterModule("EventTimer")
LibStub("AceTimer-3.0"):Embed(EventTimer)

EventTimer.Events = {}
EventTimer.Events[178019] = { ID = 178019, Name = Locale.TIMER_COLLECT_EMPOWERED_ORE, Title = Locale.TIMER_EMPOWERED_ORE, MapX = 0.57, MapY = 0.7 }
EventTimer.Events[178021] = { ID = 178021, Name = Locale.TIMER_OGRE_FIRES, Title = Locale.TIMER_OGRE_FIRES, MapX = 0.61, MapY = 0.31 }
EventTimer.Events[178020] = { ID = 178020, Name = Locale.TIMER_RISEN_SPIRITS, Title = Locale.TIMER_RISEN_SPIRITS, MapX = 0.36, MapY = 0.63 }
EventTimer.Events[178022] = { ID = 178022, Name = Locale.TIMER_STADIUM_RACING, Title = Locale.TIMER_STADIUM_RACING, MapX = 0.36, MapY = 0.36 }
EventTimer.Events[186255] = { ID = 186255, Name = Locale.TIMER_COLLECT_APEXIS_MARKS, Title = Locale.TIMER_APEXIS_MARKS }

local factionGroup, _ = UnitFactionGroup("player")
if (factionGroup == "Alliance") then
	EventTimer.Events[186255].MapX = 0.45
	EventTimer.Events[186255].MapY = 0.63
else
	EventTimer.Events[186255].MapX = 0.47
	EventTimer.Events[186255].MapY = 0.39
end

EventTimer.States = {}
EventTimer.States.Hidden = { ID = -1, Category = 6, Value = 0, MinValue = 0, MaxValue = 0, Red = 0, Green = 0, Blue = 0, Text = "", Message = nil }
EventTimer.States.Inactive = { ID = 0, Category = 5, Value = 0, MinValue = 0, MaxValue = 120, Red = 0.5, Green = 0.5, Blue = 0.5, Text = nil, Message = Locale.TIMER_ON_COOLDOWN }
EventTimer.States.Intermission = { ID = 1, Category = 4, Value = 0, MinValue = 0, MaxValue = 420, Red = 1, Green = 0, Blue = 0, Text = nil, Message = Locale.TIMER_INTERMISSION }
EventTimer.States.Announced = { ID = 2, Category = 3, Value = 0, MinValue = 0, MaxValue = 180, Red = 0.26, Green = 0.42, Blue = 1, Text = nil, Message = Locale.TIMER_ANNOUNCED }
EventTimer.States.PreActive = { ID = 3, Category = 2, Value = 60, MinValue = 0, MaxValue = 60, Red = 0, Green = 1, Blue = 0, Text = "", Message = Locale.TIMER_ACTIVE }
EventTimer.States.Active = { ID = 4, Category = 1, Value = 60, MinValue = 0, MaxValue = 60, Red = 0, Green = 1, Blue = 0, Text = "", Message = Locale.TIMER_ACTIVE }

EventTimer.IsStackRequired = false

local LandmarkTextureIndex = 60

-- Initialize the module.
function EventTimer:OnInitialize()
	Settings:RegisterDefaults(self.Name, {
		enabled = true,
		locked = false,
		showTimer = true,
		stacked = true,
		growUp = false,
		point = "CENTER",
		relativePoint = "CENTER",
		x = 0,
		y = 0
	})

	self:InitializeComponent(220, 65)
end

-- Called when the module is enabled.
function EventTimer:OnEnable()
	local settings = Settings:Get(self.Name)

	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "OnChatMessageMonsterEmote")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChangedNewArea")
	
	settings.stacked = true
	if (not settings.locked) then
		EventTimer:Unlock()
	end
	self.Component:Show()
end

-- Called when the module is disabled.
function EventTimer:OnDisable()
	self:UnregisterAllEvents()
	for i = 1, #EventTimer.statusBars do
		local statusBar = self.statusBars[i]
		statusBar:SetState(EventTimer.States.Hidden)
	end
	self.Component:Hide()
end

function EventTimer:OnChatMessageMonsterEmote(event, message, sender, language, channelString, target, flags, unused1, channelNumber, channelName, unused2, counter)
	if (Api:IsInAshran()) then
		for i = 1, #self.statusBars do
			local statusBar = self.statusBars[i]
			if (message:find(statusBar:GetEventID())) then
				statusBar:SetState(EventTimer.States.Announced)
				statusBar.clean = true
				
				for j = 1, #self.statusBars do
					if (self.statusBars[j] ~= statusBar) then
						self.statusBars[j]:SetState(EventTimer.States.Hidden)
					end
				end
			end
		end
	end
end

function EventTimer:OnZoneChangedNewArea(event)
	if (Api:IsInAshran()) then
		self.Component:Show()
	else
		local settings = Settings:Get(self.Name)
		if (settings.locked == false) then
			self:Unlock()
		else
			self.Component:Hide()
		end
		for i = 1, #EventTimer.statusBars do
			local statusBar = self.statusBars[i]
			statusBar:SetState(EventTimer.States.Hidden)
		end
	end
end

function EventTimer:OnPlayerEnteringWorld(event)
	if (Api:IsInAshran()) then
		self.Component:Show()
	else
		local settings = Settings:Get(self.Name)
		if (settings.locked == false) then
			self:Unlock()
		else
			self.Component:Hide()
		end
	end
end

function EventTimer:OnUpdate(elapsed)
	if (not Api:IsInAshran()) then
		return
	end

	-- Compensate for delays on the update such as a loading screen or reload ui
	if (elapsed > 1.5) then
		debug("OnUpdate was suspended for", elapsed, "secs")
		for i = 1, #EventTimer.statusBars do
			local statusBar = EventTimer.statusBars[i]
			if (statusBar:GetState() ~= EventTimer.States.Hidden) then
				statusBar:SetValue(statusBar:GetValue() + elapsed)
				statusBar.clean = false
			end
		end
	end
	
	-- Detect if an event has become active or was active and has just finished
	for i = 1, #EventTimer.statusBars do
		local statusBar = EventTimer.statusBars[i]
		local event = EventTimer.Events[statusBar:GetEventID()]
		local exists = EventTimer:FindWorldMapLandmark(event.Name, event.MapX, event.MapY)
		if (exists) then
			statusBar:SetState(EventTimer.States.Active)
			statusBar.clean = true
		elseif (not exists and statusBar:GetState() == EventTimer.States.Active) then
			statusBar:SetState(EventTimer.States.Inactive)
		end
	end
	
	-- Stack the status bars so the active one is at the top
	local settings = Settings:Get("EventTimer")
	if (EventTimer.IsStackRequired and settings.stacked) then
		local bars = {}
		for i = 1, #EventTimer.statusBars do
			bars[#bars + 1] = { EventTimer.statusBars[i]:GetOrder(), EventTimer.statusBars[i] }
		end
		table.sort(bars, function(a, b) return a[1] < b[1] end)
		
		local anchor = "TOPLEFT"
		local y = -5
		for key, value in pairs(bars) do
			value[2]:ClearAllPoints()
			value[2]:SetPoint("TOPLEFT", EventTimer.Component, anchor, 5, y)
			y = y - 15
		end
		
		EventTimer.IsStackRequired = false
	end
	
	-- Stack back in default order
	if (not settings.stacked) then
		local y = -5
		for i = 1, #EventTimer.statusBars do
			EventTimer.statusBars[i]:ClearAllPoints()
			EventTimer.statusBars[i]:SetPoint("TOPLEFT", EventTimer.Component, "TOPLEFT", 5, y)
			y = y - 15
		end
	end
end

function EventTimer:OnStatusBarClick(button)
	local state = self:GetState()
	local minValue, maxValue = self:GetMinMaxValues()
	local timeLeft = maxValue - self:GetValue()
	local timeText = string.format(Locale.ADDON_TIME_FORMAT, timeLeft / 60 % 60, timeLeft % 60)
	
	local message = ""
	if (self:GetState() == EventTimer.States.Inactive) then
		message = Locale.TIMER_ALL_EVENTS
	elseif (self:GetState() == EventTimer.States.Intermission) then
		message = Locale.TIMER_A_RANDOM_EVENT
	else
		message = GetSpellLink(self:GetEventID())
	end
	message = string.format(Locale.TIMER_CHAT_PREFIX, message)
	message = string.format(state.Message, message, timeText)

	if (button == "LeftButton" and IsShiftKeyDown() and IsInRaid()) then
		SendChatMessage(message, "INSTANCE_CHAT")
	end
	if (button == "LeftButton" and IsAltKeyDown() and IsInRaid()) then
		SendChatMessage(message, "RAID_WARNING")
	end
	if (button == "LeftButton" and IsControlKeyDown()) then
		local index = GetChannelName(Locale.ADDON_GENERAL_CHANNEL_NAME)
		if (index ~= nil) then 
			SendChatMessage(message, "CHANNEL", nil, index)
		end
	end
end

function EventTimer:OnTimerCallback(statusBar, transition)
	local value = statusBar:GetValue()
	local minValue, maxValue = statusBar:GetMinMaxValues()
	local timeLeft = maxValue - value
	
	statusBar:SetEventTime(string.format(Locale.ADDON_TIME_FORMAT, timeLeft / 60 % 60, timeLeft % 60))
	statusBar:SetValue(value + 1)

	if (value >= maxValue) then
		statusBar:SetState(transition)
	end
end

function EventTimer:FindWorldMapLandmark(eventName, mapX, mapY)
	local value = false

	SetMapByID(Api.AreaID)
	local count = GetNumMapLandmarks()
	for i = 1, count do
		local name, description, textureIndex, x, y, mapLinkID, inBattleMap, graveyardID, areaID, poiID, isObjectIcon = GetMapLandmarkInfo(i)
		if (textureIndex == LandmarkTextureIndex and mapX == math.roundup(x, 2) and mapY == math.roundup(y, 2)) then
			value = true
		end
	end
	SetMapToCurrentZone()

	return value
end